@@ -1,9 +1,13 @@
-# This file is Copyright (c) 2015-2019 Florent Kermarrec <florent@enjoy-digital.fr>
-# License: BSD
+#
+# This file is part of LiteSATA.
+#
+# Copyright (c) 2015-2019 Florent Kermarrec <florent@enjoy-digital.fr>
+# SPDX-License-Identifier: BSD-2-Clause
 
 from math import ceil
 
 from migen import *
+from migen.genlib.cdc import PulseSynchronizer
 
 from litex.soc.interconnect import stream
 from litex.soc.interconnect.stream import *
@@ -12,34 +16,36 @@ from litex.soc.interconnect.packet import Header, HeaderField
 # Bitrates/Frequencies -----------------------------------------------------------------------------
 
 bitrates = {
-    "sata_gen3": 6.0,
-    "sata_gen2": 3.0,
-    "sata_gen1": 1.5,
+    "gen3": 6.0,
+    "gen2": 3.0,
+    "gen1": 1.5,
 }
 
 frequencies = {
-    "sata_gen3": 150.0,
-    "sata_gen2": 75.0,
-    "sata_gen1": 37.5,
+    "gen3": 150.0,
+    "gen2": 75.0,
+    "gen1": 37.5,
 }
 
 
 # PHY / Link Layers --------------------------------------------------------------------------------
+
 primitives = {
-    "ALIGN": 0x7b4a4abc,
-    "CONT":  0x9999aa7c,
-    "SYNC":  0xb5b5957c,
-    "R_RDY": 0x4a4a957c,
-    "R_OK":  0x3535b57c,
-    "R_ERR": 0x5656b57c,
-    "R_IP":  0x5555b57c,
-    "X_RDY": 0x5757b57c,
-    "CONT":  0x9999aa7c,
-    "WTRM":  0x5858b57c,
-    "SOF":   0x3737b57c,
-    "EOF":   0xd5d5b57c,
-    "HOLD":  0xd5d5aa7c,
-    "HOLDA": 0x9595aa7c
+    "ALIGN":   0x7b4a4abc,
+    "ALIGN_N": 0x7bb5b5bc,
+    "CONT":    0x9999aa7c,
+    "SYNC":    0xb5b5957c,
+    "R_RDY":   0x4a4a957c,
+    "R_OK":    0x3535b57c,
+    "R_ERR":   0x5656b57c,
+    "R_IP":    0x5555b57c,
+    "X_RDY":   0x5757b57c,
+    "CONT":    0x9999aa7c,
+    "WTRM":    0x5858b57c,
+    "SOF":     0x3737b57c,
+    "EOF":     0xd5d5b57c,
+    "HOLD":    0xd5d5aa7c,
+    "HOLDA":   0x9595aa7c
 }
 
 def is_primitive(dword):
@@ -71,8 +77,22 @@ def link_description(dw):
     ]
     return EndpointDescription(layout)
 
+class _PulseSynchronizer(PulseSynchronizer):
+    def __init__(self, i, idomain, o, odomain):
+        PulseSynchronizer.__init__(self, idomain, odomain)
+        self.comb += [
+            self.i.eq(i),
+            o.eq(self.o)
+        ]
+
+class _RisingEdge(Module):
+    def __init__(self, i, o):
+        i_d = Signal()
+        self.sync += i_d.eq(i)
+        self.comb += o.eq(i & ~i_d)
 
 # Transport Layer ----------------------------------------------------------------------------------
+
 fis_max_dwords = 2048
 
 fis_types = {
@@ -199,6 +219,7 @@ def transport_rx_description(dw):
 
 
 # Command Layer ------------------------------------------------------------------------------------
+
 regs = {
     "WRITE_DMA_EXT":   0x35,
     "READ_DMA_EXT":    0x25,
@@ -260,6 +281,7 @@ def command_rx_data_description(dw):
 
 
 # HDD ----------------------------------------------------------------------------------------------
+
 logical_sector_size = 512  # constant since all HDDs use this
 
 def dwords2sectors(n):
